/*
    cobbler-enroll - Cobbler enrollment tool
    Copyright (C) 2011 Canonical Ltd.

    Authors: Dave Walker (Daviey) <DaveWalker@ubuntu.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Usage: ./cobbler-enroll --serverurl=http://cobbler.server/cobbler_api \
             --username=cobbler --password=cobbler --name=test \
             --profile=ubuntu-server-x86_64
            optionally, '--netif=<network I/F>' can also be passed, restricting
            registration to this I/F. If no --netif, then all I/Fs will be
            registered.

*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdbool.h>

#include <xmlrpc-c/base.h>
#include <xmlrpc-c/client.h>

#include <net/if.h>
#include <sys/ioctl.h>
#include <netinet/if_ether.h>

#include <getopt.h>

#include "cobbler-enroll.h"
#include "cobbler-xmlrpc.h"

int
main (int const argc, char **argv)
{
  int rc;
  xmlrpc_env env;
  xmlrpc_value *resultP = NULL;
  const char *token     = NULL;
  const char *system    = NULL;
  const char *serverUrl = NULL;
  const char *username  = NULL;
  const char *password  = NULL;
  const char *name      = NULL;
  const char *profile   = NULL;
  const char *netif     = NULL;

  struct cobbler_client *enroller;
  enroller = (struct cobbler_client *) malloc(sizeof(struct cobbler_client));
  init_client(enroller);
  int c;

  //int this_option_optind = optind ? optind : 1;
  int option_index = 0;

  const char* const short_options = "hds:u:p:n:P:i:";
  static struct option long_options[] = {
    {"serverurl", 1, 0, 's'},
    {"username", 1, 0, 'u'},
    {"password", 1, 0, 'p'},
    {"name", 1, 0, 'n'},
    {"profile", 1, 0, 'P'},
    {"netif", 1, 0, 'i'},
    {"help", 0, 0, 'h'},
    //{"default-interface", 1, 0, 0},
    {0, 0, 0, 0}
  };

  if (getenv("COBBLER_ENROLL_DEBUG"))
    debug = 1;

  while (1)
    {
      c = getopt_long (argc, argv, short_options, long_options, &option_index);
      if (c == -1)
        break;
      if (c == '?') {
        help(argv[0]);
        exit(1);
      }

      switch (c)
        {
        case '?':
          help(argv[0]);
          exit(1);

        case 's':
          enroller->serverUrl = optarg;
          break;

        case 'u':
          enroller->username = optarg;
          break;

        case 'p':
          enroller->password = optarg;
          break;

        case 'n':
          enroller->name = optarg;
          break;

        case 'P':
          enroller->profile = optarg;
          break;

        case 'i':
          enroller->netif = optarg;
          break;

        case 'h':
          help(argv[0]);
          exit(0);
          break;

        default:
          break;
        }
    }

  if (optind < argc)
    {
      printf ("[%s]non-option ARGV-elements: ", argv[0]);
      while (optind < argc)
      {
        printf ("%s ", argv[optind++]);
      }
      printf ("\n");
    }

  //if (argc-1 > 0) {
  //    fprintf(stderr, "This program has no arguments\n");
  //    exit(1);
  //}

/* check if all required parameters have been passed  */

  if ((enroller->serverUrl == NULL) ||
      (enroller->username == NULL) ||
      (enroller->password == NULL) ||
      (enroller->name == NULL) ||
      (enroller->profile == NULL))
  {
    fprintf(stderr, "[%s] ERROR: not all required parameters were passed\n", argv[0]);
    exit (1);
  }


  if ((rc = get_mac_address(enroller)) != 0) {
    printf("ERROR: Could not determine mac address for any interface on system.\n");
    exit(1);
  }

  if (debug)
    display_config(enroller);

  /* Initialize our error-handling environment. */
  if ((rc = init_xmlrpc_env(enroller)) != 0)
    goto out;
  if ((rc = init_xmlrpc_client(enroller)) != 0)
    goto out;
  if ((rc = cobbler_login(enroller)) != 0)
    goto out;
  if ((rc =  cobbler_register_system(enroller)) != 0)
    goto out;
  if ((rc = cobbler_save_system(enroller)) != 0)
    goto out;

/*  int fd;
  struct if_nameindex *curif, *ifs;
  char mac_str[25];
  char mod_if_cmd[100];

  int ioctl_data = 0;
  struct ifreq ifr;

  xmlrpc_value *form_value;
  xmlrpc_value *form2send;
  if ((fd = socket (PF_INET, SOCK_DGRAM, 0)) != -1)
    {
      ifs = if_nameindex ();
      for (curif = ifs; curif && curif->if_name; curif++)
        {
          if (debug)
            printf ("[%s] netif=%s., curif->if_name=%s.\n", argv[0], netif, curif->if_name);
          if ((netif == NULL) ||
              (strcasecmp (netif, curif->if_name) == 0))
          {
            form2send = xmlrpc_struct_new (&env);


            ioctl_data = socket (PF_INET, SOCK_STREAM, 0);
            memset (&ifr, 0, sizeof (ifr));

            strncpy (ifr.ifr_name, curif->if_name, sizeof (ifr.ifr_name));
            ioctl (ioctl_data, SIOCGIFHWADDR, &ifr);
            //close(ioctl_data);

            ether_ntoa_r ((const struct ether_addr *)
                          &(ifr.ifr_hwaddr.sa_data), mac_str);


            form_value = xmlrpc_build_value (&env, "s", mac_str);
            sprintf (mod_if_cmd, "macaddress-%s", ifr.ifr_name);
            xmlrpc_struct_set_value (&env, form2send, mod_if_cmd, form_value);        // set key/value in structure


            resultP = xmlrpc_client_call (&env, serverUrl, "modify_system",
                                          "(ssSs)", system,
                                          "modify_interface", form2send,
                                          token);
            xmlrpc_read_string (&env, a, &system);
            if (debug)
              printf("[%s] %s: [%s] \n", argv[0], curif->if_name, mac_str);
            if (netif != NULL) // one Net I/F requested, and we got it
              break;
        }
      }
    }

  resultP = xmlrpc_client_call (&env, serverUrl, "save_system",
                                "(ss)", system, token);
  */
  /* Clean up our error-handling environment. */

  /* Shutdown our XML-RPC client library. */

out:
  if (debug)
    printf("Exit: %d\n", rc);
  xmlrpc_env_clean(&enroller->env);
  return rc;
}

